@@ -1,4 +1,5 @@
 import sys
+from typing import Dict
 import numpy as np
 import collections
 from sklearn.neighbors import NearestNeighbors
@@ -16,7 +17,7 @@ except ModuleNotFoundError:
 try:
     import faiss
     available_engines.add("faiss")
-except ModuleNotFoundError:
+except Exception:
     print ("faiss not found")
     faiss = None
 try:
@@ -202,17 +203,24 @@ class SciKitNearestNeighbors:
 
 
 class RedisIndex:
-    def __init__(self, space, dim, index_factory=None,redis_credentials=None,max_elements=1024, ef_construction=200, M=16):
+    def __init__(self, space, dim, index_factory=None,redis_credentials=None,max_elements=1024, ef_construction=200, M=16, overwrite=False):
         self.space = space
         self.dim = dim
         self.max_elements = max_elements
         self.ef_construction = ef_construction
         self.M = M
+        if index_factory is None:
+            index_factory = "idx"
+        self.index_name = index_factory
         if redis_credentials is None:
             raise Exception("Redis credentials must be provided")
         self.redis = Redis(**redis_credentials)
         self.pipe = None
+        if overwrite:
+            self.redis.ft(self.index_name).dropindex(delete_documents=True)
         self.init_hnsw()
+        # applicable only for user events
+        self.user_keys=[]
     
     def __enter__(self):
         self.pipe = self.redis.pipeline()
@@ -227,24 +235,32 @@ class RedisIndex:
     def  __itemgetter__(self, item):
         return super().get_items([item])[0]
 
+    def user_keys(self):
+        return self.redis.keys("user:*")
+
+    def item_keys(self):
+        return self.redis.keys("item:*")
+
+    def vector_keys(self):
+        return self.redis.keys("vec:*")
+
     def search(self, data, k=1,partition=None):
         query_vector = np.array(data).astype(np.float32).tobytes()
-
         #prepare the query
         p = "(@partition:{"+partition+"})" if partition is not None else "*"
-        q = Query(f'{p}=>[KNN {k} embedding $vec_param AS vector_score]').sort_by('vector_score').paging(0,k).return_fields('vector_score','item_id').dialect(2)
+        q = Query(f'{p}=>[KNN {k} @embedding $vec_param AS vector_score]').sort_by('vector_score').paging(0,k).return_fields('vector_score','item_id').dialect(2)
         params_dict = {"vec_param": query_vector}
-        results = self.redis.ft().search(q, query_params = params_dict)
+        results = self.redis.ft(self.index_name).search(q, query_params = params_dict)
         scores, ids = [], []
         for item in results.docs:
             scores.append(item.vector_score)
             ids.append(item.item_id)
-        return (scores, ids)
+        return scores, ids
 
     def add_items(self, data, ids=None, partition=None):
         self.pipe = self.redis.pipeline(transaction=False)
         if partition is None:
-            partition=""
+            partition="NONE"
         for datum, id in zip(data, ids):
             key='item:'+ str(id)
             emb = np.array(datum).astype(np.float32).tobytes()
@@ -256,18 +272,76 @@ class RedisIndex:
     def get_items(self, ids=None):
         ret = []
         for id in ids:
-            ret.append(self.redis.hget("item:"+str(id), "embedding"))
-        return ret
+            ret.append(np.frombuffer(self.redis.hget("item:"+str(id), "embedding"), dtype=np.float32))
+        return np.vstack(ret)
+
+    def add_user_event(self, user_id: str, data: Dict[str, str],ttl: int = 60*60*24):
+        if not any(self.user_keys):
+            raise Exception("User keys must be set before adding user events")
+        vals = []
+        for key in self.user_keys:
+            v = data.get(key,"")
+            # ad hoc int trimming
+            try:
+                if v==int(v):
+                    v=int(v)
+            except:
+                pass
+            vals.append(v)
+        val = '|'.join(map(str, vals))
+        if self.pipe:
+            self.pipe.rpush("user:"+str(user_id), val)
+            if ttl:
+                self.pipe.expire("user:"+str(user_id), ttl)
+        else:
+            self.redis.rpush("user:"+str(user_id), val)
+            if ttl:
+                self.redis.expire("user:"+str(user_id), ttl)
+        return self
+    def del_user(self, user_id):
+        if self.pipe:
+            self.pipe.delete("user:"+str(user_id))
+        else:
+            self.redis.delete("user:"+str(user_id))
+    
+    def get_user_events(self, user_id: str):
+        if not any(self.user_keys):
+            raise Exception("User keys must be set before getting user events")
+        ret = self.redis.lrange("user:"+str(user_id), 0, -1)
+        return [dict(zip(self.user_keys,x.decode().split('|'))) for x in ret]
+    
+    def set_vector(self, key, arr):
+        emb = np.array(arr).astype(np.float32).tobytes()
+        self.redis.set("vec:"+str(key), emb)
+        return self
+    
+    def get_vector(self, key):
+        return np.frombuffer(self.redis.get("vec:"+str(key)), dtype=np.float32)
+
 
     def init_hnsw(self, **kwargs):
-        self.redis.ft().create_index([
+        self.redis.ft(self.index_name).create_index([
         VectorField("embedding", "HNSW", {"TYPE": "FLOAT32", "DIM": self.dim, "DISTANCE_METRIC": self.space, "INITIAL_CAP": self.max_elements, "M": self.M, "EF_CONSTRUCTION": self.ef_construction}),
         TextField("item_id"),
         TagField("partition")
         ])  
 
     def get_current_count(self):
-        raise NotImplementedError("RedisIndex is not implemented yet")
+        return int(self.redis.ft(self.index_name).info()["num_docs"])
 
     def get_max_elements(self):
-        return self.max_elements
\ No newline at end of file
+        return self.max_elements
+    
+    def info(self):
+        return self.redis.ft(self.index_name).info()
+
+if __name__=="__main__":
+    # docker run -p 6379:6379 redislabs/redisearch:2.4.5
+    sim = RedisIndex(space='cosine',dim=32,redis_credentials={"host":"127.0.0.1", "port": 6379}, overwrite=True)
+    data=np.random.random((100,32))
+    aids=["a"+str(1+i) for i in range(100)]
+    bids=["b"+str(101+i) for i in range(100)]
+    sim.add_items(data,aids,partition="a")
+    sim.add_items(data,bids,partition="b")
+    # print(sim.search(data[0],k=10,partition=None))
+    # print(sim.get_items(aids[:10]))
\ No newline at end of file
